#include "cnfaction.h"
#include "planner.h"

/*********************************************************************
 Conditional effect implementation
**********************************************************************/
Effect::Effect()
{

}
Effect::~Effect()
{

}

void Effect::set_body(Literals bd) { 
  Literals::iterator it;
  Literal l;
  
  m_body = bd;

  m_neg_body.clear();

  for (it = bd.begin(); it != bd.end(); ++it) {
    NEGATE(*it,l);
    m_neg_body.insert(l);
  }  
}

void Effect::set_head(Literals hd) {
  Literals::iterator it;
	Literal l;
	if (hd.size() == 2) {
		it = hd.begin();
		NEGATE(*it, l);
		it++;
		if (*it == l) {
			m_head.insert(*hd.begin());
			m_nondet = true;
			return;
		}
	}

	m_head = hd;    
	m_nondet = false;  
}

bool Effect::nondet() const
{
	return m_nondet;
}

const Literals* Effect::get_body() const {
  return &m_body;
}

const Literals* Effect::get_neg_body() const {
  return &m_neg_body;
}

const Literals* Effect::get_head() const {
  return &m_head;
}

/*********************************************************************
 Action implementation
**********************************************************************/
/* constructor and destructor */
Action::Action(const Planner* planner, const string& name) 
//Action::Action(const string& name) 
{
  m_planner = planner;
  m_name = name;  
}

Action::~Action()
{
  
}

/* get & set functions */
void Action::set_name(const string& name) {
  m_name = name;
}

string Action::get_name() const
{
  return m_name;
}

/* main functions */
void Action::add_exec(const Literals ex)
{
	Literals::iterator itl;
	Literal l;
	Literals s;
	m_execs.push_back(ex);


	for (itl = ex.begin(); itl != ex.end(); itl++) {
		NEGATE(*itl,l);
		s.insert(l);
	}
	m_neg_execs.insert(s);
}

void Action::add_imposs(const Literals imp)
{
  m_imposs.push_back(imp);
}

void Action::add_effect(const Literals hd, const Literals bd)
{
  Effect ef;

  ef.set_head(hd); 
  ef.set_body(bd);

  m_effects.push_back(ef);
}

const EffectList* Action::get_effects() const
{
  return &m_effects;
}


//For sensing actions
const Literal Action::lit() const
{
  return *m_effects.begin()->get_head()->begin();
}


const ExecList* Action::get_execs() const
{
  return &m_execs;
}

const ExecList* Action::get_imposs() const
{
  return &m_imposs;
}

const set<Literals>* Action::get_neg_execs() const
{
  return &m_neg_execs;
}

bool Action::is_executable(const CState* cs) const
{  
  list<Literals>::const_iterator it;
  Literals x;
  int i;

  // check impossibility conditions
  for (it = get_imposs()->begin(); 
       it != get_imposs()->end(); ++it) {
    // if the body of an impossiblity is empty
    // then the action is non-executable
    if (it->empty()) {	  
      return false;
    }

    x = m_planner->negate(*it);
    if (!cs->m_L->intersect(&x)) {      
      return false;
    }
  }

  // default is executable
  if (get_execs()->empty()) {
    return true;
  }
    
  for (it = get_execs()->begin(); it != get_execs()->end(); ++it) {
    if (cs->includes(&(*it))) {
      return true;
    }  
  }
  
  if (cs->implies(this))
		return true;
	    
  return false;
}


void Action::print() const
{
  list<Literals>::const_iterator it1;
  set<Literals>::const_iterator it3;
  EffectList::const_iterator it2;

  cout << get_name() << endl;
  // print executability condition
  cout << "    Executability:\n";
  for (it1 = get_execs()->begin(); it1 != get_execs()->end(); ++it1) {
    cout << "\t";
    m_planner->print(*it1);
    cout << endl;
  }

  /*cout << "    Neg-Executability:\n";
  for (it3 = get_neg_execs()->begin(); it3 != get_neg_execs()->end(); ++it3) {
    cout << "\t";
    m_planner->print(*it3);
    cout << endl;
  }
  */

  cout << "    Imposibility:\n";
  for (it1 = get_imposs()->begin(); 
       it1 != get_imposs()->end(); ++it1) {
    cout << "\t";
    m_planner->print(*it1);
    cout << endl;
  }

  // print conditional effects
  cout << "    Conditional Effects\n";	
  for (it2 = get_effects()->begin(); 
       it2 != get_effects()->end(); ++it2) {
    cout << "\t";
    m_planner->print(*it2->get_head());
    cout << " <- ";
    m_planner->print(*it2->get_body());
    cout << endl;	  
  }
}
