#ifndef ACTION_H
#define ACTION_H

#include "define.h"
#include "cnfstate.h"

class Planner;

/***********************************************************************
 class: Action
************************************************************************/
class Action {
 protected:
  const Planner* m_planner;
  string m_name;          // action name
  ExecList m_execs;       // executability conditions
  set<Literals> m_neg_execs;
  EffectList m_effects;   // conditional effects
  ExecList m_imposs;      // impossibility conditions

 public:
  /* constructor & destructor */
  Action(const Planner* planner, const string& name);
  //Action(const string& name);
  ~Action();

  bool m_sensing;		  // True for sensing action

  /* get/set functions */
  string get_name() const;
  void set_name(const string& name);

  const EffectList* get_effects() const;
  const Literal lit() const; //Return the literal of a sensing action
  const ExecList* get_execs() const;
  const ExecList* get_imposs() const;
  const set<Literals>* get_neg_execs() const;

  void add_exec(const Literals ex);
  void add_imposs(const Literals imp);
  void add_effect(const Literals hd, const Literals bd);

  /* main functions */  
  bool is_executable(const CState* cs) const;  
  void print() const;
};

typedef vector<Action> ActionList;

#endif
