#ifndef CNFSOLVER_H
#define CNFSOLVER_H

#include <ctime>
#include <cstring>
#include <stdint.h>
#include <errno.h>

#include <signal.h>
#include <zlib.h>
#include "Solver.h"
#include "cnfstate.h"
#include "parsingFunctions.h"

class CState;

// DIMACS Parser:

#define CHUNK_LIMIT 1048576
/*
class StreamBuffer1 {
    gzFile  in;
    char    buf[CHUNK_LIMIT];
    int     pos;
    int     size;

    void assureLookahead() {
        if (pos >= size) {
            pos  = 0;
            size = gzread(in, buf, sizeof(buf)); } }

public:
    StreamBuffer1(gzFile i) : in(i), pos(0), size(0) {
        assureLookahead(); }

    int  operator *  () { return (pos >= size) ? EOF : buf[pos]; }
    void operator ++ () { pos++; assureLookahead(); }
};
*/
//- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

template<class B>
static void skipWhitespace(B& in);

template<class B>
static void skipLine(B& in);

template<class B>
static int parseInt(B& in);

template<class B>
static void readClause(B& in, Solver& S, vec<Lit>& lits);

template<class B>
static bool match(B& in, char* str);


template<class B>
static void parse_DIMACS_main(B& in, Solver& S);
// Inserts problem into solver.
//
static void parse_DIMACS(gzFile input_stream, Solver& S);

//=================================================================================================


void printStats(Solver& solver);


static void SIGINT_handler(int signum);

//=================================================================================================
// Main:

void printUsage(char** argv);

const char* hasPrefix(const char* str, const char* prefix);


int main_solver(int argc, char** argv);

//=======================SON=======================================================================
void readClause1(const Literals& x, Solver& S, vec<Lit>& lits);
void readClause1(const Literals& x, Solver& S, vec<Lit>& lits,const boost::dynamic_bitset<>& m_L);
void readUnitClause(const Literal& l, Solver& S, vec<Lit>& lits);


//=======================Revival CNF Simplification=======================================================================
#define cpaLit(lit) (lit&1?(2*lit_var(lit)+1):2*lit_var(lit))

void readClause(const Literals& x, veci* lits);
void readUnitClause(const Literal& x, veci* lits);
bool parse_CNFtoSimp(CState* cs, solver* s);
bool parse_CNFtoSimp(set<Literals>* cnf, solver* s);
bool simplifyCNF(CState* cs);
bool simplifyCNF(set<Literals>* cs);
bool CNF_extract(CState* cs, solver* s, int Unitclauses_new, int clauses_new, int lits_new);
void CNF_extract(set<Literals>* cnf, solver* s , int Unitclauses_new, int clauses_new, int lits_new);
bool subsumes(const Literals& s, const Literals& s1);
void getNegationCNF(set<Literals>* cnf_in, set<Literals>* cnf_out);

//===================================================================================================
#define simulate_removeClause(s,c)     s_removeClause(s,c),c=NULL
#define simulate_addItAgain(s,place,c) s_addItAgain(s,c),place=c

#endif